@@ -68,7 +68,6 @@ public slots:
   void showParameterMenu();
 };
 
-class ElementParametersOld;
 class ElementParameters;
 class Parameter : public QObject
 {
@@ -87,8 +86,6 @@ public:
   };
   Parameter(ModelInstance::Element *pElement, bool defaultValue, ElementParameters *pElementParameters);
   ModelInstance::Element* getModelInstanceElement() {return mpModelInstanceElement;}
-  bool isParameter() const;
-  bool isInput() const;
   void setTab(QString tab) {mTab = tab;}
   const StringAnnotation &getTab() {return mTab;}
   void setGroup(QString group) {mGroup = group;}
@@ -109,7 +106,7 @@ public:
   QString getOriginalFixedValue() const {return mOriginalFixedValue;}
   FinalEachToolButton *getFixedFinalEachMenu() const {return mpFixedFinalEachMenuButton;}
   void setValueType(ValueType valueType) {mValueType = valueType;}
-  void setValueWidget(QString value, bool defaultValue, QString fromUnit, bool valueModified = false, bool adjustSize = true, bool unitComboBoxChanged = false);
+  void setValueWidget(QString value, bool defaultValue, QString fromUnit, bool valueModified = false, bool unitComboBoxChanged = false);
   bool isEnumeration() const {return mValueType == Enumeration;}
   bool isReplaceableComponent() const {return mValueType == ReplaceableComponent;}
   bool isReplaceableClass() const {return mValueType == ReplaceableClass;}
@@ -191,6 +188,7 @@ private:
 
   void createEditClassButton();
   void createValueWidget();
+  void createValueComboBox();
   void enableDisableUnitComboBox(const QString &value);
   void updateValueBinding(const FlatModelica::Expression expression);
   bool isValueModifiedHelper() const;
@@ -203,6 +201,7 @@ public slots:
   void unitComboBoxChanged(int index);
   void valueComboBoxChanged(int index);
   void valueCheckBoxChanged(bool toggle);
+  void valueTextBoxChanged(const QString &text);
   void showFixedMenu();
   void trueFixedClicked();
   void falseFixedClicked();
@@ -249,6 +248,7 @@ public:
   ElementParameters(ModelInstance::Element *pElement, GraphicsView *pGraphicsView, bool inherited, bool nested, ModelInstance::Modifier *pDefaultElementModifier,
                     ModelInstance::Modifier *pReplaceableConstrainedByModifier, ModelInstance::Modifier *pElementModifier, QWidget *pParent = 0);
   ~ElementParameters();
+  QString getElementQualifiedName() const;
   QString getElementParentClassName() const;
   QString getComponentClassName() const;
   QString getComponentClassComment() const;
@@ -258,6 +258,7 @@ public:
   bool isElementArray() const {return mpElement->getDimensions().isArray();}
   QString getElementDimensions() const {return mpElement->getDimensions().getTypedDimensionsString();}
   bool isInherited() const {return mInherited;}
+  bool isNested() const {return mNested;}
   QString getModification() const {return mModification;}
   void applyFinalStartFixedAndDisplayUnitModifiers(Parameter *pParameter, ModelInstance::Modifier *pModifier, bool defaultValue, bool isElementModification);
   void updateParameters();
@@ -293,6 +294,7 @@ private:
 
   void setUpDialog();
   void createTabsGroupBoxesAndParameters(ModelInstance::Model *pModelInstance, bool defaultValue);
+  void addOrUpdateParametersScrollArea(Parameter *pParameter);
   void fetchElementExtendsModifiers(ModelInstance::Model *pModelInstance, bool defaultValue);
   void fetchModifiers(ModelInstance::Modifier *pModifier);
   void fetchRootElementModifiers(ModelInstance::Element *pModelElement);
